/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Provide a directory of extensions
 * 
 * Copyright (C) 1999, Mark Spencer
 *
 * Mark Spencer <markster@linux-support.net>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */
 
#include <asterisk/lock.h>
#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/options.h>
#include <asterisk/module.h>
#include <asterisk/config.h>
#include <asterisk/say.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <pthread.h>
#include <stdio.h>
#include "../asterisk.h"
#include "../astconf.h"
#include <mysql/mysql.h>

static char *tdesc = "Extension Directory - mySQL";
static char *app = "Directory2";

static char *synopsis = "Provide directory of voicemail extensions";
static char *descrip =
"  Directory(context): Presents the user with a directory of extensions from\n"
"which they  may  select  by name. The  list  of  names  and  extensions  is\n"
"discovered from  voicemail.conf. The  context  argument  is  required,  and\n"
"specifies  the  context  in  which to interpret the extensions. Returns 0\n"
"unless the user hangs up. It  also sets up the channel on exit to enter the\n"
"extension the user selected.\n";

/* The directory will read the database settings out of voicemail.conf just like
   voicemail2 will do. */

#define DIRECTORY_CONFIG "voicemail.conf"

/* How many digits to read in */
#define NUMDIGITS 3

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

MYSQL *dbhandler=NULL;
ast_mutex_t mysqllock;
char dbuser[80];
char dbpass[80];
char dbhost[80];
char dbname[80];

static int mysql_login(void)
{
	ast_verbose( VERBOSE_PREFIX_3 "Logging into database with user %s, password %s, and database %s\n", dbuser, dbpass, dbname);
	dbhandler=mysql_init(NULL);
	if (!mysql_real_connect(dbhandler, dbhost[0] ? dbhost : NULL, dbuser, dbpass, dbname, 0, NULL, 0)) {
		ast_log(LOG_WARNING, "Error Logging into database\n");
		return(-1);
	}
	ast_mutex_init(&mysqllock);
	return(0);
}

static void mysql_logout(void)
{
	mysql_close(dbhandler);
}

static char *convert_reverse(char *ext)
{
	char *res;
	char res2[255]="";
	int lcount=0;
	res = "";
	while((ext[lcount] > 32) && lcount < NUMDIGITS) {
		switch(ext[lcount]) {
			case '1':
				sprintf(res2,"%s[1]",res);
				break;
			case '2':
				sprintf(res2,"%s[2ABCabc]",res);
				break;
			case '3':
				sprintf(res2,"%s[3DEFdef]",res);
				break;
			case '4':
				sprintf(res2,"%s[4GHIghi]",res);
				break;
			case '5':
				sprintf(res2,"%s[5JKLjkl]",res);
				break;
			case '6':
				sprintf(res2,"%s[6MNOmno]",res);
				break;
			case '7':
				sprintf(res2,"%s[7PQRSpqrs]",res);
				break;
			case '8':
				sprintf(res2,"%s[8TUVtuv]",res);
				break;
			case '9':
				sprintf(res2,"%s[9WXYZwxyz]",res);
				break;
			case '0':
				sprintf(res2,"%s[0]",res);
				break;
		}
		lcount++;
		res=res2;
	}
	res+='\0';
	return res;
}		


static int do_directory(struct ast_channel *chan, struct ast_config *cfg, char *context, char digit)
{
	/* Read in the first three digits..  "digit" is the first digit, already read */

	char ext[NUMDIGITS + 1];
	int found=0;
	int selected=0;
	char fn[256];
	char fn2[256];
	char test2[256];
	int res;
	char context2[256];
	char mailbox[256];
	MYSQL_RES *result;
	MYSQL_ROW rowval;
	MYSQL_FIELD *fields;
	int numFields;
	char query[240];
	if (!context || !strlen(context)) {
		ast_log(LOG_WARNING, "Directory must be called with an argument (context in which to interpret extensions)\n");
		return -1;
	}
	memset(ext, 0, sizeof(ext));
	ext[0] = digit;
        res = 0;
	if (ast_readstring(chan, ext + 1, NUMDIGITS - 1, 3000, 3000, "#") < 0) res = -1;

	if (!res) {
		/* Search for all names which start with those digits */
		strcpy(test2,convert_reverse(ext));
		sprintf(query, "SELECT context,mailbox FROM users WHERE (fullname RLIKE ' %s' ) OR ( fullname RLIKE '%s' NOT RLIKE ' ')",test2,test2);

		mysql_query(dbhandler,query);
		if ((result=mysql_store_result(dbhandler))==NULL){
			ast_log(LOG_WARNING, "Directory2 query failed.\n");
		}
		while((rowval=mysql_fetch_row(result))!=NULL) {
			numFields=mysql_num_fields(result);
			fields=mysql_fetch_fields(result);

			/* Inside this loop means we have a non-null result from the query */

			/* Move the results into local strings */
			strcpy(context2,rowval[0]);
			strcpy(mailbox,rowval[1]);
			
			found++;

			/* Check for the VoiceMail2 greeting first */
			snprintf(fn, sizeof(fn), "%s/voicemail/%s/%s/greet", (char *)ast_config_AST_SPOOL_DIR, context2,mailbox );

			/* Otherwise, check for an old-style Voicemail greeting */
			snprintf(fn2, sizeof(fn2), "%s/vm/%s/greet", (char *)ast_config_AST_SPOOL_DIR, mailbox);

			if (ast_fileexists(fn, NULL, chan->language) > 0) {
				res = ast_streamfile(chan, fn, chan->language);
				if (!res)
					res = ast_waitstream(chan, AST_DIGIT_ANY);
				ast_stopstream(chan);
			} else if (ast_fileexists(fn2, NULL, chan->language) > 0) {
				res = ast_streamfile(chan, fn2, chan->language);
				if (!res)
					res = ast_waitstream(chan, AST_DIGIT_ANY);
				ast_stopstream(chan);
			} else {
				res = ast_say_digit_str(chan, mailbox, AST_DIGIT_ANY, chan->language);
			}
ahem:
			if (!res)
				res = ast_streamfile(chan, "dir-instr", chan->language);
			if (!res)
				res = ast_waitstream(chan, AST_DIGIT_ANY);
			if (!res)
				res = ast_waitfordigit(chan, 3000);
			ast_stopstream(chan);
			if (res > -1) {
				if (res == '1') {
					strncpy(chan->exten, mailbox, sizeof(chan->exten)-1);
					chan->priority = 0;
					strncpy(chan->context, context2, sizeof(chan->context)-1);
					selected = 1;
					res = 0;
					break;
				} else if (res == '*') {
					res = 0;
				} else {
					res = 0;
					goto ahem;
				}
			}
			
		} // while
		mysql_free_result(result);
		if(!selected) {
			if (found) 
				res = ast_streamfile(chan, "dir-nomore", chan->language);
			else 
				res = ast_streamfile(chan, "dir-nomatch", chan->language);
			if (!res)
				res = 1;
			return res;
		}

	}
	return res;
}

static int load_config(void)
{
	char *s;
	struct ast_config *cfg;
	cfg = ast_load(DIRECTORY_CONFIG);
	if (!cfg) {
		ast_log(LOG_WARNING, "Unable to open directory configuration %s\n", DIRECTORY_CONFIG);
		return -1;
	}
	if (!(s=ast_variable_retrieve(cfg, "general", "dbuser"))) {
		strcpy(dbuser, "test");
	} else {	
		strcpy(dbuser, s);
	}
	if (!(s=ast_variable_retrieve(cfg, "general", "dbpass"))) {
		strcpy(dbpass, "test");
	} else {
		strcpy(dbpass, s);
	}
	if (!(s=ast_variable_retrieve(cfg, "general", "dbhost"))) {
		strcpy(dbhost, "");
	} else {
		strcpy(dbhost, s);
	}
	if (!(s=ast_variable_retrieve(cfg, "general", "dbname"))) {
		strcpy(dbname, "vmdb");
	} else {
		strcpy(dbname, s);
	}
	return 0;
}

static int directory_exec(struct ast_channel *chan, void *data)
{
	int res = 0;
	struct localuser *u;
			
	struct ast_config *cfg;
	cfg = ast_load(DIRECTORY_CONFIG);
	if (!cfg) {
		ast_log(LOG_WARNING, "Unable to open directory configuration %s\n", DIRECTORY_CONFIG);
		return -1;
	}
	LOCAL_USER_ADD(u);
top:
	if (!res)
		res = ast_streamfile(chan, "dir-intro", chan->language);
	if (!res)
		res = ast_waitstream(chan, AST_DIGIT_ANY);
	ast_stopstream(chan);
	if (!res)
		res = ast_waitfordigit(chan, 5000);
	if (res > 0) {
		res = do_directory(chan, cfg, (char *)data, res);
		if (res > 0) {
			res = ast_waitstream(chan, AST_DIGIT_ANY);
			ast_stopstream(chan);
			if (res >= 0) {
				goto top;
			}
		}
	}
	ast_destroy(cfg);
	LOCAL_USER_REMOVE(u);
	return res;
}

int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	mysql_logout();
	return ast_unregister_application(app);
}

int load_module(void)
{
	int res;
	res = ast_register_application(app, directory_exec, synopsis, descrip);
	if((res = load_config())) {
		return res;
	}
	if ((res = mysql_login())) {
		return res;
	}
	return res;
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
